@@ -1,4 +1,4 @@
-// Copyright (C) 2016  Stefan Vargyas
+// Copyright (C) 2016, 2017  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -95,7 +95,7 @@ uchar_t bit_set_sizes[256] = {
 size_t bit_set_get_size(
     const struct bit_set_t* set)
 {
-    size_t r = 0;
+    size_t m, r = 0;
 
     // stev: the 'bit_set_sizes' table
     // above was generated on an 8-bit
@@ -106,8 +106,10 @@ size_t bit_set_get_size(
     // using the 'main' function below!
     STATIC(CHAR_BIT == 8);
 
+    m = BIT_SET_VAL_MASK_(set);
+
     if (BIT_SET_IS_VAL_(set)) {
-        uintptr_t v = set->val;
+        uintptr_t v = set->u.val & m;
         size_t i;
 
         // stev: expecting the compiler
@@ -123,18 +125,24 @@ size_t bit_set_get_size(
     }
     else {
         const uchar_t *p, *e;
+        size_t n;
 
-        for (p = set->ptr,
-             e = p + BIT_SET_POS_END_(set);
+        n = BIT_SET_POS_END_(set);
+        ASSERT_SIZE_DEC_NO_OVERFLOW(n);
+
+        for (p = set->u.ptr,
+             e = p + (n - 1);
              p < e;
              p ++)
              r += bit_set_sizes[*p];
+
+        r += bit_set_sizes[(*p) & m];
     }
 
     return r;
 }
 
-void bit_set_print(
+static void bit_set_print_bits(
     const struct bit_set_t* set, FILE* file)
 {
     size_t i;
@@ -145,18 +153,42 @@ void bit_set_print(
     }
 }
 
+void bit_set_print(
+    const struct bit_set_t* set, FILE* file)
+{
+    if (set == NULL)
+        fputs("null", file);
+    else {
+        fprintf(file, "{\"size\":%zu,\"bits\":\"",
+            set->size);
+        bit_set_print_bits(set, file);
+        fputs("\"}", file);
+    }
+}
+
 #ifdef JSON_DEBUG
 
 void bit_set_print_debug(
     const struct bit_set_t* set, FILE* file)
 {
-    fprintf(file, "{.size=%zu .bits=\"", set->size);
-    bit_set_print(set, file);
-    fputs("\"}", file);
+    if (set == NULL)
+        fputs("(nil)", file);
+    else {
+        fprintf(file, "{.size=%zu .bits=\"",
+            set->size);
+        bit_set_print_bits(set, file);
+        fputs("\"}", file);
+    }
 }
 
 #endif
 
+size_t bit_set_get_ptr_count(
+    const struct bit_set_t* set)
+{
+    return !BIT_SET_IS_VAL_(set);
+}
+
 #else // MAIN
 
 #define BIT_SET_SIZE_LEN (SZ(1) << CHAR_BIT)
