@@ -392,8 +392,8 @@ size_t bit_set_get_ptr_count(
 #define BIT_SET_IS_AND_EQUAL(b, c)          \
     BIT_SET_IS_BIN_OP_EQUAL_(b, c, &)
 
-#define BIT_SET_DECL_AUTO(n, v, s)          \
-    struct n ## _bit_set_t                  \
+#define BIT_SET_DECL_AUTO(v, s)             \
+    struct                                  \
     {                                       \
         struct bit_set_t set;               \
         uchar_t ptr[BIT_SET_PTR_SIZE(s)];   \
