@@ -1,4 +1,4 @@
-// Copyright (C) 2016  Stefan Vargyas
+// Copyright (C) 2016, 2017  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -153,20 +153,80 @@ struct json_type_list_node_t
 };
 
 // the object value:
-//   `{ "type": "dict", "args": [ $arg* ] }'
+//   `{ "type": "dict", "args": [ $arg* ] (, "expr": "$expr" )? }'
 // where $arg is the object value:
 //   `{ "name": $string, "type": $type }'
 // where:
 //   $string is any string value and
-//   $type is any type object
+//   $type is any type object,
+// and where $expr obeys the rules:
+//   expr : bits ( "||" bits )*
+//        ;
+//   bits : term ( "|" term )*
+//        ;
+//   term : prim ( [ "&" ] prim )*
+//        ;
+//   prim : "~" prim
+//        | "(" bits ")"
+//        | KEY
+//        | NUM
+//        ;
+//   KEY  : "`" ( [^`] | "``" )+ "`"
+//        | [a-zA-Z_][0-9a-zA-Z_]*
+//        ;
+//   NUM  : [01]
+//        ;
+
+enum json_type_dict_expr_op_t
+{
+    json_type_dict_expr_op_alt,
+    json_type_dict_expr_op_or,
+    json_type_dict_expr_op_and,
+    json_type_dict_expr_op_not,
+};
+
+enum json_type_dict_expr_num_t
+{
+    json_type_dict_expr_num_zero,
+    json_type_dict_expr_num_one,
+};
+
+enum json_type_dict_expr_node_type_t
+{
+    json_type_dict_expr_node_key_type,
+    json_type_dict_expr_node_num_type,
+    json_type_dict_expr_node_op_type,
+};
+
+struct json_type_dict_expr_node_t
+{
+    enum json_type_dict_expr_node_type_t type;
+    union {
+        const uchar_t*                   key;
+        enum json_type_dict_expr_num_t   num;
+        enum json_type_dict_expr_op_t    op;
+    } node;
+    struct json_text_pos_t pos;
+};
+
+struct json_type_dict_expr_t
+{
+    const uchar_t* text;
+    const struct json_type_dict_expr_node_t* nodes;
+    size_t size;
+};
 
 #define json_type_dict_node_arg_t json_type_object_node_arg_t
-#define json_type_dict_node_t     json_type_object_node_t
+
+struct json_type_dict_node_t
+{
+    const struct json_type_dict_expr_t* expr;
+    const struct json_type_dict_node_arg_t* args;
+    size_t size;
+};
 
 struct json_type_list_attr_t;
-struct json_type_dict_trie_t;
-
-#define json_type_dict_attr_t json_type_dict_trie_t
+struct json_type_dict_attr_t;
 
 struct json_type_node_t
 {
@@ -259,6 +319,17 @@ struct json_type_list_attr_t
 #undef  TRIE_SYM_TYPE
 #undef  TRIE_NAME
 
+#define BITSET_NEED_STRUCT_ONLY
+#include "bit-set.h"
+#undef  BITSET_NEED_STRUCT_ONLY
+
+struct json_type_dict_attr_t
+{
+    const struct json_type_dict_trie_t* args;
+    const struct bit_set_t**            expr;
+    size_t                              size;
+};
+
 enum json_type_def_type_t
 {
     json_type_def_node_type,
